[<Microsoft.FSharp.Core.AutoOpen>]
module Degenz.Types

open System

[<Measure>]
type mins

[<Measure>]
type GBT

type ItemId =
    | Virus = 0
    | RemoteAccess = 1
    | Worm = 2
    | Firewall = 6
    | Encryption = 7
    | Cypher = 8

type StatId =
    | Strength = 0
    | Focus = 1
    | Luck = 2
    | Charisma = 3

type Stat = {
    Id : StatId
    BaseDecayRate : single
    BaseMinMax : Range
}

type ActiveStat = {
    Id : StatId
    Amount : int
}

module PlayerStats =
    let Strength = { Id = StatId.Strength ; BaseDecayRate = 5.0f ; BaseMinMax = Range(0, 100) }
    let Focus    = { Id = StatId.Focus ; BaseDecayRate = 5.0f ; BaseMinMax = Range(0, 100) }
    let Luck     = { Id = StatId.Luck ; BaseDecayRate = 5.0f ; BaseMinMax = Range(0, 100) }
    let Charisma = { Id = StatId.Charisma ; BaseDecayRate = 5.0f ; BaseMinMax = Range(0, 100) }

    let stats = [| Strength ; Focus ; Luck ; Charisma |]
    let statConsumableMap =
        [ ( StatId.Strength , 12 )
          ( StatId.Focus , 13 )
          ( StatId.Luck , 14 )
          ( StatId.Charisma , 15 ) ]

type HackResult =
    | Strong
    | Weak

[<CLIMutable>]
type DiscordPlayer = { Id: uint64; Name: string }
    with static member empty = { Id = 0uL ; Name = "None" }

type HackEvent = {
    IsInstigator : bool
    Adversary : DiscordPlayer
    Success : bool
    HackId : int
}

type PlayerEventType =
    | Hacking of HackEvent
    | Shielding of shieldId : int
    | Stealing of instigator : bool * adversary : DiscordPlayer
    | Imprison

type PlayerEvent =
    { Type : PlayerEventType
      Cooldown : int<mins>
      Timestamp : DateTime }

type HackItem = Item * int * int * int<mins>
and  ShieldItem = Item * int * int<mins>
and  FoodItem = Item * (Item -> PlayerData -> PlayerData)
and  AccessoryItem = Item * (Item -> PlayerData -> PlayerData)
and  ItemType =
    | Hack of power : int * hackClass : int * cooldown : int<mins>
    | Shield of shieldClass : int * cooldown : int<mins>
    | Food of effect : (Item -> PlayerData -> PlayerData)
    | Accessory of effect : (Item -> PlayerData -> PlayerData)
and Item = {
    Id : int
    Name : string
    Price : int<GBT>
    Type : ItemType
}
and Inventory = Item list
and PlayerData = {
    DiscordId : uint64
    Name : string
    Inventory : Inventory
    Events : PlayerEvent array
    Stats : ActiveStat list
    Bank : int<GBT>
}
//        Achievements : string array
//        XP : int
with member this.basicPlayer = { Id = this.DiscordId ; Name = this.Name }
     static member empty =
         { DiscordId = 0uL
           Name = "None"
           Inventory = []
           Events = [||]
           Stats = []
//               Achievements = [||]
//               XP = 0
           Bank = 0<GBT> }
